from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server import Server
from bot_graphql.abc.data_query_abc import DataQueryABC
from bot_graphql.abc.filter_abc import FilterABC
from bot_graphql.filter.auto_role_filter import AutoRoleFilter
from bot_graphql.filter.level_filter import LevelFilter
from bot_graphql.filter.user_filter import UserFilter


class ServerQuery(DataQueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        auto_roles: AutoRoleRepositoryABC,
        clients: ClientRepositoryABC,
        users: UserRepositoryABC,
        levels: LevelRepositoryABC,
    ):
        DataQueryABC.__init__(self, "Server")

        self._bot = bot
        self._auto_roles = auto_roles
        self._users = users
        self._clients = clients
        self._users = users
        self._levels = levels

        self.set_field("id", self.resolve_id)
        self.set_field("discord_id", self.resolve_discord_id)
        self.set_field("name", self.resolve_name)
        self.set_field("auto_roles", self.resolve_auto_roles)
        self.set_field("clients", self.resolve_clients)
        self.set_field("members", self.resolve_members)
        self.set_field("levels", self.resolve_levels)

    @staticmethod
    def resolve_id(server: Server, *_):
        return server.server_id

    @staticmethod
    def resolve_discord_id(server: Server, *_):
        return server.discord_server_id

    def resolve_name(self, server: Server, *_):
        guild = self._bot.get_guild(server.discord_server_id)
        return None if guild is None else guild.name

    def resolve_auto_roles(self, server: Server, *_, filter: AutoRoleFilter = None):
        if filter is not None:
            return filter.filter(self._auto_roles.get_auto_roles_by_server_id(server.server_id))

        return self._auto_roles.get_auto_roles_by_server_id(server.server_id)

    def resolve_clients(self, server: Server, *_):
        return self._clients.get_clients_by_server_id(server.server_id)

    @FilterABC.resolve_filter_annotation
    def resolve_members(self, server: Server, *_, filter: UserFilter = None):
        if filter is not None:
            return filter.filter(self._users.get_users_by_server_id(server.server_id))

        return self._users.get_users_by_server_id(server.server_id)

    @FilterABC.resolve_filter_annotation
    def resolve_levels(self, server: Server, *_, filter: LevelFilter = None):
        if filter is not None:
            return filter.filter(self._levels.get_levels_by_server_id(server.server_id))

        return self._levels.get_levels_by_server_id(server.server_id)
